#!/bin/bash
printf "Starting php profiler installer...\n\n"
EXT_PKG_DIR="$(pwd)/"
echo "Setting Package Directory: ${EXT_PKG_DIR}"


SUPPORTED_VERSIONS=("5.6" "7.0" "7.1" "7.2" "7.3" "7.4" "8.0" "8.1" "8.2")

# glob all php patterns
PHP_DIR_LIST=(
  "/usr/local"
  "/opt"
  "/usr/bin"
  "/Applications/XAMPP/" # osx
)

phpbinaries=()
phpresults=()

if [[ $# -ge 1 ]]; then
  PHP_DIR_LIST=()
  for DIR_OR_FILE in "$@"; do
      if [[ ! -z "$DIR_OR_FILE" ]]; then
        PHP_BIN_INPUT_FILE_OR_DIR=$DIR_OR_FILE
        if [[ -d "$PHP_BIN_INPUT_FILE_OR_DIR" ]]; then
          PHP_DIR_LIST+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom folder set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi

        if [[ -f "$PHP_BIN_INPUT_FILE_OR_DIR" ]] && [[ -x $PHP_BIN_INPUT_FILE_OR_DIR ]]; then
          phpbinaries+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom binary set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi
      fi
  done
fi


for php_dir in ${PHP_DIR_LIST[@]}; do
  if [[ -d "$php_dir" ]]; then
    echo "Checking folder at $php_dir"
    phpresults+=($(find -L "$php_dir" -type f -iregex ".*\/php[@]*[0-9]*[.]*[0-9]*" 2> /dev/null))
  fi
done

is_supported_version () {
  # $1 PHP binary
  for ver in ${SUPPORTED_VERSIONS[@]}; do
    if [[ $1 = $ver ]]; then
      echo 0
      return
    fi
  done

  echo 1
}

# glob php versions in files
for PHP_RES in ${phpresults[@]}; do
	# check if we support it
  if [[ -f $PHP_RES ]] && [[ -x $PHP_RES ]]; then
    if [[ $(is_supported_version $($PHP_RES -n -r "echo PHP_MAJOR_VERSION.'.'.PHP_MINOR_VERSION;")) ]]; then
      # a dd to list of php binaries
      if [[ -L $PHP_RES ]]; then
        PHP_RES_LINK=$(readlink $PHP_RES)
        if [[ -f $PHP_RES_LINK ]] && [[ -x $PHP_RES_LINK ]]; then
          echo "Found binary (link): $PHP_RES_LINK"
          phpbinaries+=($PHP_RES_LINK)
        fi
      else
        echo "Found binary: $PHP_RES"
        phpbinaries+=($PHP_RES)
      fi
    else
      echo "Not found binary: $PHP_RES"
    fi
  fi
done

printf "Removing duplicates...\n\n"
deduped=($(echo ${phpbinaries[*]} | tr ' ' '\n' | sort -u))
if [[ ${#deduped[*]} -eq 0 ]]; then
     echo "Unable to find PHP installation."
     exit 1
fi

# setup extension in php binary's extension dir
for phpbin in ${deduped[@]}; do
    echo "Using binary: $phpbin"
    CURRENT_DIR=$(pwd)
    EXTENSION_DIR=$($phpbin -r 'echo "[".ini_get("extension_dir")."]";')
    EXTENSION_DIR=${EXTENSION_DIR/#*\[}
    EXTENSION_DIR=${EXTENSION_DIR/%\]}
    PHP_MAJOR_VERSION=$($phpbin -n -r 'echo PHP_MAJOR_VERSION;')
    PHP_MINOR_VERSION=$($phpbin -n -r 'echo PHP_MINOR_VERSION;')
    PHP_ZTS_ENABLED=$($phpbin -n -r 'echo PHP_ZTS;')
    PHP_DEBUG_ENABLED=$($phpbin -n -r 'echo PHP_DEBUG;')

    PHP_VERSION="${PHP_MAJOR_VERSION}.${PHP_MINOR_VERSION}";
    if [[ "$PHP_ZTS_ENABLED" == "1" ]]; then
        PHP_VERSION="${PHP_VERSION}.zts";
    fi

    if [[ ! -d "$EXTENSION_DIR" ]]; then
      echo "Unable to find extension location: ${PHP_VERSION}\n\n"
      continue
    fi

    echo "Detected PHP ${PHP_VERSION}" 
    printf "Found extension location: ${EXTENSION_DIR}\n\n"
    EXTENSION_FILE="${EXT_PKG_DIR}macos/stackify-${PHP_VERSION}.so"
    echo "Using package extension: ${EXTENSION_FILE}"
    echo "Using package wrapper: ${EXT_PKG_DIR}/Stackify.php"

    if [[ -f "${EXTENSION_DIR}/stackify.so" ]]; then
      echo "PHP Extension: ${EXTENSION_DIR}/stackify.so - Already exists"
      rm "${EXTENSION_DIR}/stackify.so"
    fi

    if [[ -f "${EXTENSION_DIR}/Stackify.php" ]]; then
      echo "PHP Extension Wrapper: ${EXTENSION_DIR}/Stackify.php - Already exists"
      rm "${EXTENSION_DIR}/Stackify.php"
    fi

    if [[ -f "$EXTENSION_FILE" ]]; then
	      mkdir -p "${EXTENSION_DIR}"

        unlink "${EXTENSION_DIR}/stackify.so" > /dev/null 2>&1
        unlink "${EXTENSION_DIR}/Stackify.php" > /dev/null 2>&1
        cp "${EXTENSION_FILE}" "${EXTENSION_DIR}/stackify.so"
        cp "${EXT_PKG_DIR}/Stackify.php" "${EXTENSION_DIR}/Stackify.php"
        chmod 775 ${EXTENSION_DIR}/stackify.so
        chmod 775 ${EXTENSION_DIR}/Stackify.php

        echo "Added extension: ${EXTENSION_DIR}/stackify.so"
        echo "Added file: ${EXTENSION_DIR}/Stackify.php"
    else
      echo "Stackify extension is not found. PHP Version not supported."
      echo "Extension file: ${EXTENSION_FILE}"
      echo "Unable to install extension."
      continue
    fi

    # if extension was added already
    if [[ -f "${EXTENSION_DIR}/stackify.so" ]]; then 
      if [[ ( ! $($phpbin -r 'echo "[".extension_loaded("stackify")."]";') =~ ".*\[1\].*" || ! $($phpbin -r 'echo ini_get("stackify.prefix_enabled");') =~ ".*\[1\].*" ) ]]; then
        # if our extension isn't already loaded, check for config directory
        PHP_CONFIG_FILE_SCAN_DIR=$($phpbin -r 'echo "[".PHP_CONFIG_FILE_SCAN_DIR."]";')
        PHP_CONFIG_FILE_SCAN_DIR=${PHP_CONFIG_FILE_SCAN_DIR/#*\[}
        PHP_CONFIG_FILE_SCAN_DIR=${PHP_CONFIG_FILE_SCAN_DIR/%\]}

        PHP_INI_LOCATION=$($phpbin -r 'echo "[".dirname(php_ini_loaded_file())."]";')
        PHP_INI_LOCATION=${PHP_INI_LOCATION/#*\[}
        PHP_INI_LOCATION=${PHP_INI_LOCATION/%\]}

        echo "Checking config file scan dir: $PHP_CONFIG_FILE_SCAN_DIR"
        echo "Checking ini location: $PHP_INI_LOCATION"

        if [[ $PHP_CONFIG_FILE_SCAN_DIR ]]; then
            if [[ ! -d ${PHP_CONFIG_FILE_SCAN_DIR} ]]; then
              mkdir -p ${PHP_CONFIG_FILE_SCAN_DIR}
            fi

            if [[ ! -f "${PHP_CONFIG_FILE_SCAN_DIR}/99-stackify.ini"  && ! -f "${PHP_CONFIG_FILE_SCAN_DIR}/stackify.ini" ]]; then
              # if config file scan directory is not null, install .ini there
              cp "${EXT_PKG_DIR}/stackify.ini" "${PHP_CONFIG_FILE_SCAN_DIR}/stackify.ini"
              echo "Added .ini in PHP_CONFIG_FILE_SCAN_DIR: ${PHP_CONFIG_FILE_SCAN_DIR}/stackify.ini"
            else
              echo "File .ini already exists in PHP_CONFIG_FILE_SCAN_DIR: ${PHP_CONFIG_FILE_SCAN_DIR}"
            fi
        elif [[ $PHP_INI_LOCATION == /usr/local/* ]]; then
            # else, if global php.ini is in /usr/local, install it there instead
            if [[ ! -d ${PHP_INI_LOCATION} ]]; then
              mkdir -p ${PHP_INI_LOCATION}
            fi

            if [[ ! -f "${PHP_INI_LOCATION}/stackify.ini" ]]; then
              # if config file scan directory is not null, install .ini there
              cp "${EXT_PKG_DIR}/stackify.ini" "${PHP_INI_LOCATION}/stackify.ini"
              echo "Added .ini in PHP_INI_LOCATION: ${PHP_INI_LOCATION}/stackify.ini"
            else
              echo "File .ini already exists in PHP_INI_LOCATION: ${PHP_INI_LOCATION}"
            fi
        else
            echo "Unable to access .ini location. Please go to docs.stackify.com for instructions on installing in this location."
        fi
      else
        echo "Stackify extension already loaded. Skipping .ini installation."
      fi
    else
        echo "Stackify extension is not loaded. PHP Version not supported. Skipping .ini installation."
    fi

    printf "\n"
    if [[ -f "${EXTENSION_DIR}/stackify.so" && -f "${EXTENSION_DIR}/Stackify.php" ]]; then
      echo "PHP Profiler Install Complete."
      echo "Please restart your web server."
    else
      echo "Unable to install extension."
    fi
    printf "\n"
done

exit 0