#!/bin/bash
printf "Starting php profiler uninstaller...\n\n"
EXT_PKG_DIR="$(pwd)/"
echo "Loading package directory: $EXT_PKG_DIR"
#LOGGED_IN_USER=$( echo "show State:/Users/ConsoleUser" | scutil | awk '/Name :/ && ! /loginwindow/ { print $3 }' )

SUPPORTED_VERSIONS=("5.6" "7.0" "7.1" "7.2" "7.3" "7.4" "8.0" "8.1" "8.2")

# glob all php patterns
PHP_DIR_LIST=(
  "/usr/local"
  "/opt"
  "/usr/bin"
  "/Applications/XAMPP/" # osx
)

phpbinaries=()
phpresults=()

if [[ $# -ge 1 ]]; then
  PHP_DIR_LIST=()
  for DIR_OR_FILE in "$@"; do
      if [[ ! -z "$DIR_OR_FILE" ]]; then
        PHP_BIN_INPUT_FILE_OR_DIR=$DIR_OR_FILE
        if [[ -d "$PHP_BIN_INPUT_FILE_OR_DIR" ]]; then
          PHP_DIR_LIST+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom folder set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi

        if [[ -f "$PHP_BIN_INPUT_FILE_OR_DIR" ]] && [[ -x $PHP_BIN_INPUT_FILE_OR_DIR ]]; then
          phpbinaries+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom binary set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi
      fi
  done
fi


for php_dir in ${PHP_DIR_LIST[@]}; do
  if [[ -d "$php_dir" ]]; then
    echo "Checking folder at $php_dir"
    phpresults+=($(find -L "$php_dir" -type f -iregex ".*\/php[@]*[0-9]*[.]*[0-9]*"))
  fi
done

is_supported_version () {
  # $1 PHP binary
  for ver in ${SUPPORTED_VERSIONS[@]}; do
    if [[ $1 = $ver ]]; then
      echo 0
      return
    fi
  done

  echo 1
}

# glob php versions in files
for PHP_RES in ${phpresults[@]}; do
	# check if we support it
  if [[ -f $PHP_RES ]] && [[ -x $PHP_RES ]]; then
    if [[ $(is_supported_version $($PHP_RES -n -r "echo PHP_MAJOR_VERSION.'.'.PHP_MINOR_VERSION;")) ]]; then
      # a dd to list of php binaries
      if [[ -L $PHP_RES ]]; then
        PHP_RES_LINK=$(readlink $PHP_RES)
        if [[ -f $PHP_RES_LINK ]] && [[ -x $PHP_RES_LINK ]]; then
          echo "Found binary (link): $PHP_RES_LINK"
          phpbinaries+=($PHP_RES_LINK)
        fi
      else
        echo "Found binary: $PHP_RES"
        phpbinaries+=($PHP_RES)
      fi
    else
      echo "Not found binary: $PHP_RES"
    fi
  fi
done

printf "Removing duplicates...\n\n"
deduped=($(echo ${phpbinaries[*]} | tr ' ' '\n' | sort -u))
if [[ ${#deduped[*]} -eq 0 ]]; then
     echo "Unable to find PHP installation."
     exit 1
fi

# setup extension in php binary's extension dir
for phpbin in ${deduped[@]}; do
    echo "Using binary: $phpbin"
    CURRENT_DIR=$(pwd)
    EXTENSION_DIR=$($phpbin -r 'echo "[".ini_get("extension_dir")."]";')
    EXTENSION_DIR=${EXTENSION_DIR/#*\[}
    EXTENSION_DIR=${EXTENSION_DIR/%\]}

    EXTENSION_DIR_NO_PHP_INI=$($phpbin -n -r 'echo "[".ini_get("extension_dir")."]";')
    EXTENSION_DIR_NO_PHP_INI=${EXTENSION_DIR_NO_PHP_INI/#*\[}
    EXTENSION_DIR_NO_PHP_INI=${EXTENSION_DIR_NO_PHP_INI/%\]}

    PHP_MAJOR_VERSION=$($phpbin -n -r 'echo PHP_MAJOR_VERSION;')
    PHP_MINOR_VERSION=$($phpbin -n -r 'echo PHP_MINOR_VERSION;')
    PHP_ZTS_ENABLED=$($phpbin -n -r 'echo PHP_ZTS;')
    PHP_DEBUG_ENABLED=$($phpbin -n -r 'echo PHP_DEBUG;')

    PHP_VERSION="${PHP_MAJOR_VERSION}.${PHP_MINOR_VERSION}";
    if [[ "$PHP_ZTS_ENABLED" == "1" ]]; then
        PHP_VERSION="${PHP_VERSION}.zts";
    fi

    if [[ ! -d "$EXTENSION_DIR" ]]; then
      echo "Unable to find extension location: ${PHP_VERSION} - Directory: $EXTENSION_DIR \n"

      if [[ ! -d "$EXTENSION_DIR_NO_PHP_INI" ]]; then
        echo "Unable to find extension location: ${PHP_VERSION} - Directory: $EXTENSION_DIR_NO_PHP_INI \n"
        continue
      fi
    fi

    echo "Detected PHP ${PHP_VERSION} Extension" 

    if [[ -f "${EXTENSION_DIR}/stackify.so" ]]; then
      echo "PHP Extension: ${EXTENSION_DIR}/stackify.so - exists"
      rm "${EXTENSION_DIR}/stackify.so"
    fi

    if [[ -f "${EXTENSION_DIR}/Stackify.php" ]]; then
      echo "PHP Extension Wrapper: ${EXTENSION_DIR}/Stackify.php - exists"
      rm "${EXTENSION_DIR}/Stackify.php"
    fi

    if [[ -f "${EXTENSION_DIR_NO_PHP_INI}/stackify.so" ]]; then
      echo "PHP Extension: ${EXTENSION_DIR_NO_PHP_INI}/stackify.so - exists"
      rm "${EXTENSION_DIR_NO_PHP_INI}/stackify.so"
    fi

    if [[ -f "${EXTENSION_DIR_NO_PHP_INI}/Stackify.php" ]]; then
      echo "PHP Extension Wrapper: ${EXTENSION_DIR_NO_PHP_INI}/Stackify.php - exists"
      rm "${EXTENSION_DIR_NO_PHP_INI}/Stackify.php"
    fi

    echo "Successfully removed PHP ${PHP_VERSION} Profiler"
    printf "\n"
done

exit 0